==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/Run.java <==
package de.fzj.unicore.ucc.actions;

import java.io.ByteArrayOutputStream;
import java.io.File;

import org.apache.commons.cli.OptionBuilder;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;

import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.runner.Runner;
import de.fzj.unicore.ucc.util.Builder;
import de.fzj.unicore.ucc.util.UnitParser;

/**
 * runs a job through UNICORE 6
 * 
 * @author schuller
 */
public class Run extends ActionBase {

	protected Runner runner;

	protected Builder builder;

	/**
	 * site to use
	 */
	protected String siteName=null;

	/**
	 * synchronous mode
	 */
	protected boolean synchronous;

	/**
	 * do not add job id prefixes to output file names 
	 */
	protected boolean brief;

	/**
	 * treat job file as JSDL doc instead of the standard JSON
	 */
	protected boolean isJSDL;

	/**
	 * whether the JSDL doc should be validated
	 */
	protected boolean validateJSDL=true;

	/**
	 * whether job submission to the batch system (server-side) should be
	 * scheduled for a certain time 
	 */
	protected String scheduled=null;
	
	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_MODE_LONG)
				.withDescription("Run asynchronous, writing a job ID file for use with other ucc commands")
				.isRequired(false)
				.create(OPT_MODE)
		);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Site Name")
				.withArgName("Vsite")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
		);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_NOPREFIX_LONG)
				.withDescription("Short output file names")
				.isRequired(false)
				.create(OPT_NOPREFIX)
		);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_STDOUT_LONG)
				.withDescription("Name name for storing the standard output")
				.withArgName("stdout")
				.hasArg()
				.isRequired(false)
				.create(OPT_STDOUT)
		);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_STDERR_LONG)
				.withDescription("Name name for storing the standard error")
				.withArgName("stderr")
				.hasArg()
				.isRequired(false)
				.create(OPT_STDERR)
		);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_JSDL_LONG)
				.withDescription("Treat job file as JSDL document")
				.isRequired(false)
				.create(OPT_JSDL)
		);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SAMPLE_LONG)
				.withDescription("Print an example job and quit")
				.isRequired(false)
				.create(OPT_SAMPLE)
		);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SCHEDULED_LONG)
				.withDescription("Schedule the job for a specific time (in ISO8601 format)")
				.isRequired(false)
				.hasArg()
				.create(OPT_SCHEDULED)
				);
	}


	@Override
	public String getName() {
		return "run";
	}

	@Override
	public String getArgumentList(){
		return "[<jobfile>]";
	}

	@Override
	public String getSynopsis(){
		return "Runs a job through UNICORE 6. " +
		"The job definition is read from <jobfile> or stdin. " +
		"Run 'ucc run -"+OPT_SAMPLE+"' to see an example job."+
		"A job can be executed in two modes. In the default synchronous mode, " +
		"UCC will wait for the job to finish. In asynchonous mode, initiated by the '"+OPT_MODE+"' option, " +
		"a descriptor file will be written that can be used later with other UCC commands.";
	}

	@Override
	public String getDescription(){
		return "run a job through UNICORE 6";
	}
	@Override
	public String getCommandGroup(){
		return "Job execution";
	}


	@Override
	public void process(){
		super.process();

		if(getBooleanOption(OPT_SAMPLE_LONG, OPT_SAMPLE)){
			printSampleJob();
			endProcessing(0);
		}

		siteName=getCommandLine().getOptionValue(OPT_SITENAME);

		synchronous=!getBooleanOption(OPT_MODE_LONG, OPT_MODE);
		verbose("Synchronous processing = "+synchronous);

		brief=getBooleanOption(OPT_NOPREFIX_LONG, OPT_NOPREFIX);
		verbose("Adding job id to output file names = "+!brief);

		isJSDL=getBooleanOption(OPT_JSDL_LONG, OPT_JSDL);
		verbose("Assuming JSDL job file = "+isJSDL);

		scheduled=getOption(OPT_SCHEDULED_LONG, OPT_SCHEDULED);
		if(scheduled!=null){
			scheduled=UnitParser.convertDateToISO8601(scheduled);
			verbose("Will schedule job submission for "+scheduled);
		}
		
		initBuilder();

		run();

	}

	protected void initBuilder(){
		String jobFile=null;
		if(getCommandLine().getArgs().length>1){
			jobFile=getCommandLine().getArgs()[1];
			if(isJSDL){
				try{
					verbose("Loading JSDL file.");
					builder=new Builder();
					builder.setMessageWriter(this);
					builder.setRegistry(registry);
					builder.setSecurityProperties(securityProperties);
					JobDefinitionDocument jdd=JobDefinitionDocument.Factory.parse(new File(jobFile));
					if(validateJSDL)doValidate(jdd);
					builder.setJob(jdd);
				}catch(Exception e){
					error("Can't parse jsdl file <"+jobFile+">",e);
					endProcessing(ERROR_CLIENT);
				}
			}
			else {
				try{
					builder=new Builder(new File(jobFile));
					builder.setMessageWriter(this);
					builder.setRegistry(registry);
					builder.setSecurityProperties(securityProperties);
					verbose("Read job from <"+jobFile+">");
				}catch(Exception e){
					error("Can't parse job file <"+jobFile+">",e);
					endProcessing(ERROR_CLIENT);
				}
			}
		}
		else{
			try{
				message("Reading job from stdin:");
				message("");
				ByteArrayOutputStream bos=new ByteArrayOutputStream();
				int b=0;
				while((b=System.in.read())!=-1){
					bos.write(b);
				}
				builder=new Builder(bos.toString());
			}catch(Exception e){
				error("Can't read job from stdin.",e);
				endProcessing(ERROR_CLIENT);
			}
		}
	}

	protected void run(){
		builder.setProperty("Output",output.getAbsolutePath());
		builder.setProperty("IDLocation",output.getAbsolutePath());
		builder.setProperty("KeepFinishedJob", "true");
		builder.setProperty("DetailedStatusDisplay", "true");
		if(scheduled!=null)builder.setProperty("Not before", scheduled);
		if(siteName!=null)builder.setProperty("Site", siteName);
		runner=new Runner(registry,securityProperties,builder,this);
		runner.setAsyncMode(!synchronous);
		runner.setBriefOutfileNames(brief);
		runner.setProperties(properties);
		try{
			runner.run();
			runner.dumpJobProperties();
			lastJobAddress=runner.getJob().getEPR().getAddress().getStringValue();
		}catch(RuntimeException ex){
			runner.dumpJobLog();
			endProcessing(ERROR);
		}
	}

	protected void doValidate(JobDefinitionDocument jdd){
		boolean isValid=Builder.isValidJSDL(jdd, this);
		if(!isValid && Boolean.parseBoolean(properties.getProperty(PROP_FAIL_ON_VALIDATION_ERRORS))){
			String message="Stopping due to validation errors (set property '"
				+PROP_FAIL_ON_VALIDATION_ERRORS+"=false' to ignore them)";
			throw new EndProcessingException(ERROR_JSDL_INVALID,message);
		}
	}

	public void printSampleJob(){
		message(Builder.getJobSample());
	}


	/*
	 * for unit testing
	 */
	private static String lastJobAddress;

	public static String getLastJobAddress() {
		return lastJobAddress;
	}

}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/actions/Run.java <==
package de.fzj.unicore.ucc.actions;

import java.io.ByteArrayOutputStream;
import java.io.File;

import org.apache.commons.cli.OptionBuilder;
import org.ggf.schemas.jsdl.x2005.x11.jsdl.JobDefinitionDocument;

import de.fzj.unicore.ucc.helpers.EndProcessingException;
import de.fzj.unicore.ucc.runner.Runner;
import de.fzj.unicore.ucc.util.Builder;
import de.fzj.unicore.ucc.util.UnitParser;

/**
 * runs a job through UNICORE 6
 * 
 * @author schuller
 */
public class Run extends ActionBase {

	protected Runner runner;

	protected Builder builder;

	/**
	 * site to use
	 */
	protected String siteName=null;

	/**
	 * synchronous mode
	 */
	protected boolean synchronous;

	/**
	 * do not add job id prefixes to output file names 
	 */
	protected boolean brief;

	/**
	 * treat job file as JSDL doc instead of the standard JSON
	 */
	protected boolean isJSDL;

	/**
	 * whether the JSDL doc should be validated
	 */
	protected boolean validateJSDL=true;

	/**
	 * whether job submission to the batch system (server-side) should be
	 * scheduled for a certain time 
	 */
	protected String scheduled=null;

	@Override
	@SuppressWarnings("all")
	protected void createOptions() {
		super.createOptions();

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_MODE_LONG)
				.withDescription("Run asynchronous, writing a job ID file for use with other ucc commands")
				.isRequired(false)
				.create(OPT_MODE)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SITENAME_LONG)
				.withDescription("Site Name")
				.withArgName("Vsite")
				.hasArg()
				.isRequired(false)
				.create(OPT_SITENAME)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_NOPREFIX_LONG)
				.withDescription("Short output file names")
				.isRequired(false)
				.create(OPT_NOPREFIX)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_STDOUT_LONG)
				.withDescription("Name name for storing the standard output")
				.withArgName("stdout")
				.hasArg()
				.isRequired(false)
				.create(OPT_STDOUT)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_STDERR_LONG)
				.withDescription("Name name for storing the standard error")
				.withArgName("stderr")
				.hasArg()
				.isRequired(false)
				.create(OPT_STDERR)
				);
		getOptions().addOption(OptionBuilder.withLongOpt(OPT_JSDL_LONG)
				.withDescription("Treat job file as JSDL document")
				.isRequired(false)
				.create(OPT_JSDL)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SAMPLE_LONG)
				.withDescription("Print an example job and quit")
				.isRequired(false)
				.create(OPT_SAMPLE)
				);

		getOptions().addOption(OptionBuilder.withLongOpt(OPT_SCHEDULED_LONG)
				.withDescription("Schedule the job for a specific time (in ISO8601 format)")
				.isRequired(false)
				.hasArg()
				.create(OPT_SCHEDULED)
				);
	}


	@Override
	public String getName() {
		return "run";
	}

	@Override
	public String getArgumentList(){
		return "[<jobfile>]";
	}

	@Override
	public String getSynopsis(){
		return "Runs a job through UNICORE 6. " +
				"The job definition is read from <jobfile> or stdin. " +
				"Run 'ucc run -"+OPT_SAMPLE+"' to see an example job."+
				"A job can be executed in two modes. In the default synchronous mode, " +
				"UCC will wait for the job to finish. In asynchonous mode, initiated by the '"+OPT_MODE+"' option, " +
				"a descriptor file will be written that can be used later with other UCC commands.";
	}

	@Override
	public String getDescription(){
		return "run a job through UNICORE 6";
	}
	@Override
	public String getCommandGroup(){
		return "Job execution";
	}


	@Override
	public void process(){
		super.process();

		if(getBooleanOption(OPT_SAMPLE_LONG, OPT_SAMPLE)){
			printSampleJob();
			endProcessing(0);
		}

		siteName=getCommandLine().getOptionValue(OPT_SITENAME);

		synchronous=!getBooleanOption(OPT_MODE_LONG, OPT_MODE);
		verbose("Synchronous processing = "+synchronous);

		brief=getBooleanOption(OPT_NOPREFIX_LONG, OPT_NOPREFIX);
		verbose("Adding job id to output file names = "+!brief);

		isJSDL=getBooleanOption(OPT_JSDL_LONG, OPT_JSDL);
		verbose("Assuming JSDL job file = "+isJSDL);

		scheduled=getOption(OPT_SCHEDULED_LONG, OPT_SCHEDULED);
		if(scheduled!=null){
			scheduled=UnitParser.convertDateToISO8601(scheduled);
			verbose("Will schedule job submission for "+scheduled);
		}

		if(getCommandLine().getArgs().length>1){
			for(int i=1; i<getCommandLine().getArgs().length;i++){
				initBuilder(getCommandLine().getArgs()[i]);
				run();
			}
		}
		else{
			initBuilderFromStdin();
			run();
		}

	}

	/**
	 * 
	 * @param jobFile - job file name
	 */
	protected void initBuilder(String jobFile){
		if(isJSDL){
			try{
				verbose("Loading JSDL file.");
				builder=new Builder();
				builder.setMessageWriter(this);
				builder.setRegistry(registry);
				builder.setSecurityProperties(securityProperties);
				JobDefinitionDocument jdd=JobDefinitionDocument.Factory.parse(new File(jobFile));
				if(validateJSDL)doValidate(jdd);
				builder.setJob(jdd);
			}catch(Exception e){
				error("Can't parse jsdl file <"+jobFile+">",e);
				endProcessing(ERROR_CLIENT);
			}
		}
		else {
			try{
				builder=new Builder(new File(jobFile));
				builder.setMessageWriter(this);
				builder.setRegistry(registry);
				builder.setSecurityProperties(securityProperties);
				verbose("Read job from <"+jobFile+">");
			}catch(Exception e){
				error("Can't parse job file <"+jobFile+">",e);
				endProcessing(ERROR_CLIENT);
			}
		}
	}

	protected void initBuilderFromStdin(){
		try{
			message("Reading job from stdin:");
			message("");
			ByteArrayOutputStream bos=new ByteArrayOutputStream();
			int b=0;
			while((b=System.in.read())!=-1){
				bos.write(b);
			}
			builder = new Builder(bos.toString());
		}catch(Exception e){
			error("Can't read job from stdin.",e);
			endProcessing(ERROR_CLIENT);
		}
	}

	protected void run(){
		builder.setProperty("Output",output.getAbsolutePath());
		builder.setProperty("IDLocation",output.getAbsolutePath());
		builder.setProperty("KeepFinishedJob", "true");
		builder.setProperty("DetailedStatusDisplay", "true");
		if(scheduled!=null)builder.setProperty("Not before", scheduled);
		if(siteName!=null)builder.setProperty("Site", siteName);
		runner=new Runner(registry,securityProperties,builder,this);
		runner.setAsyncMode(!synchronous);
		runner.setBriefOutfileNames(brief);
		runner.setProperties(properties);
		try{
			runner.run();
			lastJobPropertiesFile=runner.dumpJobProperties();
			lastJobAddress=runner.getJob().getEPR().getAddress().getStringValue();
			lastJobFile=builder.getProperty("jobIdFile");
		}catch(RuntimeException ex){
			runner.dumpJobLog();
			endProcessing(ERROR);
		}
	}

	protected void doValidate(JobDefinitionDocument jdd){
		boolean isValid=Builder.isValidJSDL(jdd, this);
		if(!isValid && Boolean.parseBoolean(properties.getProperty(PROP_FAIL_ON_VALIDATION_ERRORS))){
			String message="Stopping due to validation errors (set property '"
					+PROP_FAIL_ON_VALIDATION_ERRORS+"=false' to ignore them)";
			throw new EndProcessingException(ERROR_JSDL_INVALID,message);
		}
	}

	public void printSampleJob(){
		message(Builder.getJobSample());
	}


	/*
	 * for unit testing
	 */
	private static String lastJobAddress;

	public static String getLastJobAddress() {
		return lastJobAddress;
	}

	private static String lastJobFile;

	public static String getLastJobFile() {
		return lastJobFile;
	}

	private static String lastJobPropertiesFile;

	public static String getLastJobPropertiesFile() {
		return lastJobPropertiesFile;
	}
}
